﻿#########################################################################################
# Fonction Repair-HomeDir							               			            #
############################################################Edgarcia#09/16/2011#V2.0#007#

#V2.0 Refonte complete de la fonction pour l'orienter migration via CSV

Import-Module ActiveDirectory
Function global:Repair-Homedir
{
	#Parametres de la commande :
	param(
		[parameter(Mandatory=$true,ValueFromPipeline=$true,HelpMessage="Indiquez le nom de la source")]$Name,
		[parameter(Mandatory=$true,HelpMessage="Indiquez le type de source : OU, CSV, Group ou User")]$Source,
		[parameter(Mandatory=$false,HelpMessage="Indiquez la lettre de lecteur de la HomeDirectory")]$HomeLetter,
		[switch]$Right,
		[switch]$Owner
	)
			
	Process
	{
	Write-Host $Right $Owner
		Switch ($Source) # Choix de la source
		{
			OU
			{ 
				Write-Host "Traitement de l'OU $Name"
				$arrUser = (Get-ADUser -Filter * -SearchBase $Name -Properties HomeDirectory, SamAccountName)
				if (-not $arrUser)
				{
					Write-Warning "Il n'y a pas d'utilisateur dans cette OU ou le chemin de l'OU est incorrect (attention à la casse)"
					BREAK
				}
				foreach ($user in $arrUser)
				{
					repairhdir # Execution de la fonction de copie / suppression
				}
			}
			Group
			{
				Write-Host "Traitement du groupe AD $Name"
				try
				{
					$strGrp = (Get-ADGroup -Identity $Name -Properties members)
				}
				#Récupération de l'erreur en cas de groupe inexistant :
				catch [Microsoft.ActiveDirectory.Management.ADIdentityNotFoundException]
				{
					Write-warning "Le Groupe $Name n'existe pas."
					Break
				}
				
				#Traitement des membres du groupe
				Foreach ($strMember in $($strGrp.Members))
				{
					try
					{				
						$User = (Get-ADUser -Filter {(DistinguishedName -eq $strMember) -and (ObjectClass -eq "user")} -Properties Name, DistinguishedName, HomeDirectory, SamAccountName, Description, MemberOf)
					}
					catch [Microsoft.ActiveDirectory.Management.ADIdentityNotFoundException]
					{
					}
					
					repairhdir # Execution de la fonction de copie / suppression
				}
			}
			User
			{
				Write-Host "Traitement de l'utilisateur $Name"
				try
				{
					$User = (Get-ADUser $Name -Properties HomeDirectory, SamAccountName)
				}
				#Récupération de l'erreur en cas d'utilisateur inexistant :
				catch [Microsoft.ActiveDirectory.Management.ADIdentityNotFoundException]
				{
					Write-warning "L'utilisateur $Name n'existe pas."
					Break
				}
				repairhdir # Execution de la fonction de copie / suppression
			}
			CSV
			{
				$Right = $true
				$Owner = $true
				if ((Test-Path $Name) -eq $false) # Test de la présence du CSV
				{
					Write-Warning "Le CSV : $Name n'est pas accessible ou n'existe pas"
					BREAK
				}
				
				try
				{
					#Import du CSV :
					$arrCsv = (Import-Csv $Name -Delimiter ";")
				}
				#Récupération de l'erreur en cas de mauvais format CSV
				Catch
				{
					Write-warning "Il y a un problème avec le fichier CSV, vérifiez son encodage et supprimez les colonnes vides en fin de ligne (du type ;;;;;)"
					Break
				}
				
				foreach ($Line in $arrCsv)
				{
					$User = (Get-ADUser $($Line.LDAPLogin) -Properties HomeDirectory, SamAccountName)
					repairhdir # Execution de la fonction de copie / suppression
				}
			}
		}		
	}
<#
.SYNOPSIS 
Permet de réparer les HomeDirectory des utilisateurs AD et peut être exploité dans le cadre d'une migration de donnée grâce à l'utilisation du fichier CSV généré par Match-HomeDir.

.DESCRIPTION
Repair-HomeDir est une fonction qui permet de réparer les droits de l'HomeDirectory d'un utilisateur. Il rend la propriete à l'utilisateur et lui ajoute le contrôle total. La fonction réparera les droits sur le dossier indiqué dans la variable HomeDir de son profil Active Directory.
    
.PARAMETER Source
Parametre obligatoire. La source peut être : 
- User : La fonction corrigera le Home de l'utilisateur spécifié dans -Name
- CSV : Permet d'utiliser le fichier CSV généré par la fonction Match-HomeDir. Repair-HomeDir utilise les champs FolderSource, qui contient le chemin complet du Home à migrer et LDAPLogin qui doit contenir le LDAP de l'utilisateur.
- Group : cela signifie que le script cherchera les utilisateurs dans un groupe AD à préciser dans -Name
- OU : La fonction cherchera les utilisateurs dans une Unité d'Organisation Active Directory et corrigera leurs HomeDirectory

.PARAMETER Name
Parametre obligatoire. Indiquez le nom de la source, qui peut être un utilisateur (utilisez l'identifiant SamAccountName : edgarcia), un groupe d'utilisateur Active Directory, une Unité d'Organisation.
(il faut respecter scrupuleusement la casse) ou un fichier CSV généré par Match-HomeDir. En utilisant un CSV, Repair-HomeDir renommera également les HomeDir en utilisant le SamAccountName.

.PARAMETER HomeLetter
Parametre facultatif qui n'est utile que dans le cadre d'une migration via CSV. Indique la lettre de lecteur qui sera attribuée à la HomeDirectory dans le profil AD de l'utilisateur.

.PARAMETER Owner
Parametre facultatif. Corrige le proprietaire NTFS de la HomeDirectory. Implicite lorsque la source est un CSV.

.PARAMETER Right
Parametre facultatif. Corrige les droits NTFS de la HomeDirectory. Implicite lorsque la source est un CSV.

.INPUTS
None. You cannot pipe objects to Repair-HomeDir.ps1.

.OUTPUTS
La sortie peut être redirigée.

.EXAMPLE
C:\PS> Repair-HomeDir -Source User -Name edgarcia -Owner -Right
Cette commande reappliquera les droits de l'utilisateur edgarcia sur son HomeDir : contrôle total et proprietés.

.EXAMPLE
C:\PS> Repair-HomeDir -Source Group -Name AD_Grp -Owner -Right
Cette commande reappliquera les droits de tous les utilisateurs se trouvant dans le groupe Active Directory AD_Grp.

.EXAMPLE
C:\PS> Repair-HomeDir -Source OU -Name "OU=Users,DC=domaine,DC=local" -Owner
Cette commande reappliquera les droits de tous les utilisateurs se trouvant dans l'OU Active Directory Users.

.EXAMPLE
C:\PS> Repair-HomeDir -Source CSV -Name c:\match.csv -HomeLetter N:
Cette commande lit le fichier CSV (obtenu avec Match-HomDir) et renomme, remet les droits et remet le proprietaire sur toutes les HomeDirectory présentes dans le CSV.

.NOTES
Edouard Garcia, 21/06/2011
#>
}

Function global:repairhdir
{
	if ($Source -match "CSV")
	{
		$strOldNasPath = $($Line.DossierSource)
		$position = $strOldNasPath.LastIndexOfAny("\")
		$strNewNasPath = $strOldNasPath.SubString(0,$position)
		$strNewNasPath += "\$($user.SamAccountName)"
		if ($strOldNasPath -ne $strNewNasPath)
		{
				try
				{
				Rename-Item -Path $strOldNasPath -NewName $($User.SamAccountName) -Force
				}
				catch [System.Management.Automation.PSInvalidOperationException]
				{
					Write-Host "Le dossier source n'existe pas"
				}
				
				
				Write-Host "dossier identique"
		}
		
		
		$position = $strOldNasPath.LastIndexOfAny("\")
		$strNewNasPath = $strOldNasPath.SubString(0,$position)
		$strNewNasPath += "\$($user.SamAccountName)"
		Set-Aduser -identity $User -Homedir $strNewNasPath -HomeDrive $HomeLetter
		$User = (Get-ADUser $($Line.LDAPLogin) -Properties HomeDirectory, SamAccountName) # On recharge l'utilisateur avec ses nouvelles proprietes
	}	
		
		if ($User.HomeDirectory)
		{
			if ((Test-Path $($User.HomeDirectory)) -eq $true)
			{
				
				if ($Owner)
				{
					Write-Host "Traitement du proprietaire $($User.Name) sur le dossier $($User.HomeDirectory)" -ForegroundColor Green
					& C:\Windows\system32\icacls $($User.HomeDirectory) /setowner [DOMAINE]\$($user.SamAccountName) /T /C /Q
				}
				
				if ($Right)
				{
					Write-Host "Traitement des droits de l'utilisateur $($User.Name) sur le dossier $($User.HomeDirectory)" -ForegroundColor Green
					& C:\Windows\system32\icacls $($User.HomeDirectory) /grant [DOMAINE]\$($user.SamAccountName):`(OI`)`(CI`)F /T /C /Q
				}
							
			}
			Else
			{
				Write-Host "Le dossier HomeDir $($User.HomeDirectory) de l'utilisateur $($User.Name) n'existe pas" -ForegroundColor Red
			}
		}
		Else
		{
			Write-Host "L'utilisateur $($User.Name) n'a pas de HomeDir" -ForegroundColor Red
		}
	}